@@ -3,18 +3,18 @@
 //! Convolutions in this library can be defined in time or frequency domain. In
 //! frequency domain the convolution is automatically transformed into a multiplication
 //! which is the analog operation to a convolution in time domain.
-use {RealNumber, Zero};
 use numbers::*;
+use std::ops::*;
 use num_complex::{Complex32, Complex64};
 use std::marker::PhantomData;
-use std::mem;
 use vector_types::*;
-use inline_vector::{InlineVector, InternalBuffer};
-use std;
+use inline_vector::InlineVector;
+use super::FixedLenBuffer;
 
 /// A convolution function in time domain and real number space
-pub trait RealImpulseResponse<T> : Sync
-    where T: RealNumber {
+pub trait RealImpulseResponse<T>: Sync
+    where T: RealNumber
+{
     /// Indicates whether this function is symmetric around 0 or not.
     /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
     fn is_symmetric(&self) -> bool;
@@ -24,8 +24,9 @@ pub trait RealImpulseResponse<T> : Sync
 }
 
 /// A convolution function in frequency domain and real number space
-pub trait RealFrequencyResponse<T> : Sync
-    where T: RealNumber {
+pub trait RealFrequencyResponse<T>: Sync
+    where T: RealNumber
+{
     /// Indicates whether this function is symmetric around 0 or not.
     /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
     fn is_symmetric(&self) -> bool;
@@ -35,8 +36,9 @@ pub trait RealFrequencyResponse<T> : Sync
 }
 
 /// A convolution function in time domain and complex number space
-pub trait ComplexImpulseResponse<T> : Sync
-    where T: RealNumber {
+pub trait ComplexImpulseResponse<T>: Sync
+    where T: RealNumber
+{
     /// Indicates whether this function is symmetric around 0 or not.
     /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
     fn is_symmetric(&self) -> bool;
@@ -46,8 +48,9 @@ pub trait ComplexImpulseResponse<T> : Sync
 }
 
 /// A convolution function in frequency domain and complex number space
-pub trait ComplexFrequencyResponse<T> : Sync
-    where T: RealNumber {
+pub trait ComplexFrequencyResponse<T>: Sync
+    where T: RealNumber
+{
     /// Indicates whether this function is symmetric around 0 or not.
     /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
     fn is_symmetric(&self) -> bool;
@@ -116,6 +119,32 @@ macro_rules! define_complex_lookup_table {
 }
 define_complex_lookup_table!(ComplexTimeLinearTableLookup; ComplexFrequencyLinearTableLookup);
 
+/// Linear interpolation between two points.
+fn linear_interpolation_between_bins<T: RealNumber, C>(x: T, round: usize, table: &[C]) -> C 
+    where C: Mul<C, Output = C> + Add<C, Output = C> + Sub<C, Output = C> + Mul<T, Output = C> + Copy {
+    let round_float = T::from(round).unwrap();
+    let len = table.len();
+    if x > round_float {
+        let other = round + 1;
+        if other >= len {
+            return table[round];
+        }
+        let y0 = table[round];
+        let x0 = round_float;
+        let y1 = table[other];
+        y0 + (y1 - y0) * (x - x0)
+    } else {
+        if round == 0 {
+            return table[round];
+        }
+        let other = round - 1;
+        let y0 = table[round];
+        let x0 = round_float;
+        let y1 = table[other];
+        y0 + (y1 - y0) * (x0 - x)
+    }
+}
+
 macro_rules! add_linear_table_lookup_impl {
     ($($name: ident: $conv_type: ident, $($data_type: ident, $result_type:ident),*);*) => {
         $(
@@ -137,29 +166,12 @@ macro_rules! add_linear_table_lookup_impl {
                         }
 
                         let round_tolerance = 1e-6;
-                        if (x - round_float).abs() < round_tolerance {
+                        let is_exactly_at_bin = (x - round_float).abs() < round_tolerance;
+                        if is_exactly_at_bin {
                             return self.table[round];
                         }
 
-                        if x > round_float {
-                            let other = round + 1;
-                            if other >= len {
-                                return self.table[round];
-                            }
-                            let y0 = self.table[round];
-                            let x0 = round_float;
-                            let y1 = self.table[other];
-                            y0 + (y1 - y0) * (x - x0)
-                        } else {
-                            if round == 0 {
-                                return self.table[round];
-                            }
-                            let other = round - 1;
-                            let y0 = self.table[round];
-                            let x0 = round_float;
-                            let y1 = self.table[other];
-                            y0 + (y1 - y0) * (x0 - x)
-                        }
+                        linear_interpolation_between_bins(x, round, &self.table[..])
                     }
                 }
 
@@ -209,8 +221,13 @@ macro_rules! add_real_linear_table_impl {
                 impl $name<$data_type> {
                     /// Convert the lookup table into complex number space
                     pub fn to_complex(&self) -> $complex<$data_type> {
-                        let vector = self.table.clone().to_real_time_vec();
-                        let mut buffer = InternalBuffer::new();
+                        let len = self.table.len();
+                        let vector = InlineVector::of_size($data_type::zero(), 2 * len);
+                        let mut vector = vector.to_real_time_vec();
+                        vector.resize(len).expect("shrinking should always succeed");
+                        &mut vector[0.. len].copy_from_slice(&self.table[..]);
+                        vector.set_delta(self.delta);
+                        let mut buffer = FixedLenBuffer::new(InlineVector::of_size($data_type::zero(), 2 * vector.len()));
                         let complex = vector.to_complex_b(&mut buffer);
                         let complex = complex.complex(..);
                         let is_symmetric = self.is_symmetric;
@@ -244,7 +261,7 @@ macro_rules! add_complex_linear_table_impl {
                         }
                         let mut vector = interleaved.to_complex_time_vec();
                         vector.set_delta(self.delta);
-                        let mut buffer = InternalBuffer::new();
+                        let mut buffer = FixedLenBuffer::new(InlineVector::of_size($data_type::zero(), complex.len()));
                         let real = vector.to_real_b(&mut buffer);
                         let real = &real[..];
                         let is_symmetric = self.is_symmetric;
@@ -277,7 +294,7 @@ macro_rules! add_complex_time_linear_table_impl {
                     }
                     let mut vector = interleaved.to_complex_time_vec();
                     vector.set_delta(self.delta);
-                    let mut buffer = InternalBuffer::new();
+                    let mut buffer = FixedLenBuffer::new(InlineVector::of_size($data_type::zero(), vector.len()));
                     let freq = vector.fft(&mut buffer);
                     let delta = freq.delta();
                     let freq = freq.complex(..);
@@ -301,11 +318,15 @@ macro_rules! add_real_time_linear_table_impl {
     ($($data_type: ident),*) => {
         $(
             impl RealTimeLinearTableLookup<$data_type> {
-/// Convert the lookup table into a magnitude spectrum
+                /// Convert the lookup table into a magnitude spectrum
                 pub fn fft(self) -> RealFrequencyLinearTableLookup<$data_type> {
-                    let mut vector = self.table.clone().to_real_time_vec();
+                    let len = self.table.len();
+                    let vector = InlineVector::of_size($data_type::zero(), 2 * len);
+                    let mut vector = vector.to_real_time_vec();
+                    vector.resize(len).expect("shrinking should always succeed");
+                    &mut vector[0.. len].copy_from_slice(&self.table[..]);
                     vector.set_delta(self.delta);
-                    let mut buffer = InternalBuffer::new();
+                    let mut buffer = FixedLenBuffer::new(InlineVector::of_size($data_type::zero(), 2 * len));
                     let freq = vector.fft(&mut buffer);
                     let freq = freq.magnitude_b(&mut buffer);
                     let is_symmetric = self.is_symmetric;
@@ -341,7 +362,7 @@ macro_rules! add_complex_frequency_linear_table_impl {
                     }
                     let mut vector = interleaved.to_complex_freq_vec();
                     vector.set_delta(self.delta);
-                    let mut buffer = InternalBuffer::new();
+                    let mut buffer = FixedLenBuffer::new(InlineVector::of_size($data_type::zero(), vector.len()));
                     let time = vector.ifft(&mut buffer);
                     let delta = time.delta();
                     let time = time.complex(..);
@@ -480,147 +501,10 @@ impl<T> SincFunction<T>
     }
 }
 
-/// A real function which can be constructed outside this crate.
-#[cfg(feature="std")]
-pub struct ForeignRealConvolutionFunction<T>
-    where T: RealNumber
-{
-    /// The function
-    pub conv_function: extern "C" fn(*const std::os::raw::c_void, T) -> T,
-
-    /// The data which is passed to the function.
-    ///
-    /// Actual data type is a `const* c_void`, but Rust doesn't allow that
-    /// because it's unsafe so we store
-    /// it as `usize` and transmute it when necessary. Callers should make
-    /// very sure safety is guaranteed.
-    pub conv_data: usize,
-
-    /// Indicates whether this function is symmetric around 0 or not.
-    /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
-    pub is_symmetric: bool,
-}
-
-#[cfg(feature="std")]
-impl<T> ForeignRealConvolutionFunction<T>
-    where T: RealNumber
-{
-    /// Creates a new real function
-    pub unsafe fn new(function: extern "C" fn(*const std::os::raw::c_void, T) -> T,
-               function_data: *const std::os::raw::c_void,
-               is_symmetric: bool)
-               -> Self {
-        ForeignRealConvolutionFunction {
-            conv_function: function,
-            conv_data: mem::transmute(function_data),
-            is_symmetric: is_symmetric,
-        }
-    }
-}
-
-#[cfg(feature="std")]
-impl<T> RealImpulseResponse<T> for ForeignRealConvolutionFunction<T>
-    where T: RealNumber
-{
-    fn is_symmetric(&self) -> bool {
-        self.is_symmetric
-    }
-
-    fn calc(&self, x: T) -> T {
-        let fun = self.conv_function;
-        fun(self.conv_data as *const std::os::raw::c_void, x)
-    }
-}
-
-#[cfg(feature="std")]
-impl<T> RealFrequencyResponse<T> for ForeignRealConvolutionFunction<T>
-    where T: RealNumber
-{
-    fn is_symmetric(&self) -> bool {
-        self.is_symmetric
-    }
-
-    fn calc(&self, x: T) -> T {
-        let fun = self.conv_function;
-        fun(self.conv_data as *const std::os::raw::c_void, x)
-    }
-}
-
-/// A complex function which can be constructed outside this crate.
-#[cfg(feature="std")]
-pub struct ForeignComplexConvolutionFunction<T>
-    where T: RealNumber
-{
-    /// The function
-    pub conv_function: extern "C" fn(*const std::os::raw::c_void, T) -> Complex<T>,
-
-    /// The data which is passed to the window function
-    ///
-    /// Actual data type is a `const* c_void`, but Rust doesn't allow that
-    /// because it's unsafe so we store
-    /// it as `usize` and transmute it when necessary. Callers should make very
-    /// sure safety is guaranteed.
-    pub conv_data: usize,
-
-    /// Indicates whether this function is symmetric around 0 or not.
-    /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
-    pub is_symmetric: bool,
-}
-
-#[cfg(feature="std")]
-impl<T> ForeignComplexConvolutionFunction<T>
-    where T: RealNumber
-{
-    /// Creates a new real function
-    pub unsafe fn new(function: extern "C" fn(*const std::os::raw::c_void, T) -> Complex<T>,
-               function_data: *const std::os::raw::c_void,
-               is_symmetric: bool)
-               -> Self {
-        ForeignComplexConvolutionFunction {
-            conv_function: function,
-            conv_data: mem::transmute(function_data),
-            is_symmetric: is_symmetric,
-        }
-    }
-}
-
-#[cfg(feature="std")]
-impl<T> ComplexImpulseResponse<T> for ForeignComplexConvolutionFunction<T>
-    where T: RealNumber
-{
-    fn is_symmetric(&self) -> bool {
-        self.is_symmetric
-    }
-
-    fn calc(&self, x: T) -> Complex<T> {
-        let fun = self.conv_function;
-        fun(self.conv_data as *const std::os::raw::c_void, x)
-    }
-}
-
-#[cfg(feature="std")]
-impl<T> ComplexFrequencyResponse<T> for ForeignComplexConvolutionFunction<T>
-    where T: RealNumber
-{
-    fn is_symmetric(&self) -> bool {
-        self.is_symmetric
-    }
-
-    /// Indicates whether this function is symmetric around 0 or not.
-    /// Symmetry is defined as `self.calc(x) == self.calc(-x)`.
-    fn calc(&self, x: T) -> Complex<T> {
-        let fun = self.conv_function;
-        fun(self.conv_data as *const std::os::raw::c_void, x)
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
-    use RealNumber;
     use std::fmt::Debug;
-    use numbers::Complex;
-    use Zero;
 
     fn conv_test<T, C>(conv: C, expected: &[T], step: T, tolerance: T)
         where T: RealNumber + Debug,
