@@ -1,23 +1,25 @@
 use std::ops::*;
 use arrayvec::*;
-use {Zero, VoidResult, ErrorReason, RealNumber, Buffer};
-use std::mem;
+use {VoidResult, ErrorReason};
+use numbers::*;
 use std::iter::FromIterator;
 use std::slice::Iter;
 
+const MAX_CAPACITY: usize = 64;
+
 /// A type which internally switches between stack and heap allocation.
 /// This is supposed to perform faster but the main reason is that this
 /// way we automatically have a limited stack allocation available on systems
 /// without heap, and on systems with heap allocation we don't have to worry
 /// about introducing artifical limits.
-///
-/// Thanks to: http://stackoverflow.com/questions/27859822/alloca-variable-length-arrays-in-rust
 pub enum InlineVector<T> {
-    Inline(ArrayVec<[T; 64]>),
+    Inline(ArrayVec<[T; MAX_CAPACITY]>),
 }
 
 impl<T> InlineVector<T>
-    where T: Copy {
+    where T: Copy
+{
+    /// Create a new vector of a given size filled with a given default value.
     pub fn of_size(default: T, n: usize) -> InlineVector<T> {
         let mut result = Self::with_capacity(n);
         for _ in 0..n {
@@ -28,17 +30,35 @@ impl<T> InlineVector<T>
     }
 }
 
+/// Most of the operations behave as defined for `std::Vec`.
 impl<T> InlineVector<T> {
-    pub fn with_capacity(n: usize) -> InlineVector<T> {
-        InlineVector::Inline(ArrayVec::<[T; 64]>::new())
+    /// Returns the maximimum size the vector can possibly have.
+    pub fn max_capacity() -> usize {
+        MAX_CAPACITY
     }
 
+    /// Returns a vector with a given capacity.
+    pub fn with_capacity(_: usize) -> InlineVector<T> {
+        InlineVector::Inline(ArrayVec::<[T; MAX_CAPACITY]>::new())
+    }
+
+    /// Returns a vector with a default capacity. The default
+    /// capacity will be relativly small and should only be used 
+    /// to store small lookup tables or results. 
+    ///
+    /// As of today the default capacity is `64`.
+    pub fn with_default_capcacity() -> InlineVector<T> {
+        Self::with_capacity(Self::max_capacity())
+    }
+
+    /// Returns a vector holding a single element.
     pub fn with_elem(elem: T) -> InlineVector<T> {
         let mut vector = Self::with_capacity(1);
         vector.push(elem);
         vector
     }
 
+    /// Returns an empty vector.
     pub fn empty() -> InlineVector<T> {
         Self::with_capacity(0)
     }
@@ -48,17 +68,22 @@ impl<T> InlineVector<T> {
             &mut InlineVector::Inline(ref mut v) => {
                 let res = v.push(elem);
                 if res.is_some() {
-                    panic!("InlineVector capacity exceeded, please open a defect against `basic_dsp`");
+                    panic!("InlineVector capacity exceeded, please open a defect against \
+                            `basic_dsp`");
                 }
-            },
+            }
         };
     }
 
+    pub fn remove(&mut self, index: usize) -> T {
+        match self {
+            &mut InlineVector::Inline(ref mut v) => v.remove(index).unwrap(),
+        }
+    }
+
     pub fn pop(&mut self) -> Option<T> {
         match self {
-            &mut InlineVector::Inline(ref mut v) => {
-                v.pop()
-            },
+            &mut InlineVector::Inline(ref mut v) => v.pop(),
         }
     }
 
@@ -74,11 +99,29 @@ impl<T> InlineVector<T> {
         }
     }
 
-	pub fn iter(&self) -> Iter<T> {
-		match self {
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
+    pub fn iter(&self) -> Iter<T> {
+        match self {
             &InlineVector::Inline(ref v) => v.iter(),
         }
-	}
+    }
+
+    pub fn append(&mut self, other: &mut Self) {
+        while !other.is_empty() {
+            self.push(other.remove(0));
+        }
+    }
+
+    pub fn insert(&mut self, index: usize, element: T) {
+        match self {
+            &mut InlineVector::Inline(ref mut v) => {
+                let _ = v.insert(index, element).unwrap();
+            }
+        }
+    }
 }
 
 impl<T: Zero + Clone> InlineVector<T> {
@@ -90,18 +133,11 @@ impl<T: Zero + Clone> InlineVector<T> {
                 } else {
                     Err(ErrorReason::TypeCanNotResize)
                 }
-            },
+            }
         }
     }
 }
 
-/// A buffer which stores a single inline vector and never shrinks.
-pub struct InternalBuffer<T>
-    where T: RealNumber
-{
-    temp: InlineVector<T>,
-}
-
 impl<T> Index<usize> for InlineVector<T> {
     type Output = T;
 
@@ -176,15 +212,15 @@ impl<T> IndexMut<RangeTo<usize>> for InlineVector<T> {
 
 impl<T: Clone> Clone for InlineVector<T> {
     fn clone(&self) -> Self {
-         match self {
+        match self {
             &InlineVector::Inline(ref v) => InlineVector::Inline(v.clone()),
         }
     }
 }
 
 impl<T> FromIterator<T> for InlineVector<T> {
-    fn from_iter<I: IntoIterator<Item=T>>(iter: I) -> Self {
-        let mut c = InlineVector::with_capacity(64);
+    fn from_iter<I: IntoIterator<Item = T>>(iter: I) -> Self {
+        let mut c = InlineVector::with_default_capcacity();
 
         for i in iter {
             c.push(i);
@@ -193,40 +229,3 @@ impl<T> FromIterator<T> for InlineVector<T> {
         c
     }
 }
-
-impl<T> InternalBuffer<T>
-    where T: RealNumber
-{
-    /// Creates a new buffer which is ready to be passed around.
-    pub fn new() -> InternalBuffer<T> {
-        InternalBuffer { temp: InlineVector::with_capacity(0) }
-    }
-}
-
-impl<T> Buffer<InlineVector<T>, T> for InternalBuffer<T>
-    where T: RealNumber
-{
-    fn get(&mut self, len: usize) -> InlineVector<T> {
-        if len <= self.temp.len() {
-            let mut result = InlineVector::with_capacity(0);
-            mem::swap(&mut result, &mut self.temp);
-            return result;
-        }
-
-        InlineVector::of_size(T::zero(), len)
-    }
-
-    fn construct_new(&mut self, len: usize) -> InlineVector<T> {
-        InlineVector::of_size(T::zero(), len)
-    }
-
-    fn free(&mut self, storage: InlineVector<T>) {
-        if storage.len() > self.temp.len() {
-            self.temp = storage;
-        }
-    }
-
-    fn alloc_len(&self) -> usize {
-        self.temp.capacity()
-    }
-}
