@@ -1,11 +1,10 @@
-use RealNumber;
 use multicore_support::*;
 use simd_extensions::*;
 use numbers::*;
 use std::ops::*;
-use super::super::{Owner, ToRealResult, ErrorReason, Buffer, Vector, Resize, MetaData, ResizeOps,
-                   DspVec, ToSliceMut, ToSlice, VoidResult, Domain, ComplexNumberSpace,
-                   RededicateForceOps};
+use super::super::{ToRealResult, ErrorReason, Buffer, Vector, Resize, MetaData, DspVec,
+                   ToSliceMut, ToSlice, VoidResult, Domain, ComplexNumberSpace,
+                   RededicateForceOps, RealNumberSpace, NumberSpace, GetMetaData, PosEq};
 
 /// Defines transformations from complex to real number space.
 ///
@@ -115,7 +114,7 @@ pub trait ComplexToRealTransformsOpsBuffered<S, T>: ToRealResult
     /// assert_eq!([5.0, 5.0], result[0..]);
     /// # }
     /// ```
-    fn magnitude_b<B>(self, buffer: &mut B) -> Self::RealResult where B: Buffer<S, T>;
+    fn magnitude_b<B>(self, buffer: &mut B) -> Self::RealResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Gets the square root of the absolute value of all vector elements.
     /// # Example
@@ -131,7 +130,8 @@ pub trait ComplexToRealTransformsOpsBuffered<S, T>: ToRealResult
     /// assert_eq!([25.0, 25.0], result[0..]);
     /// # }
     /// ```
-    fn magnitude_squared_b<B>(self, buffer: &mut B) -> Self::RealResult where B: Buffer<S, T>;
+    fn magnitude_squared_b<B>(self, buffer: &mut B) -> Self::RealResult
+        where B: for<'a> Buffer<'a, S, T>;
 
     /// Gets all real elements.
     /// # Example
@@ -147,7 +147,7 @@ pub trait ComplexToRealTransformsOpsBuffered<S, T>: ToRealResult
     /// assert_eq!([1.0, 3.0], result[0..]);
     /// # }
     /// ```
-    fn to_real_b<B>(self, buffer: &mut B) -> Self::RealResult where B: Buffer<S, T>;
+    fn to_real_b<B>(self, buffer: &mut B) -> Self::RealResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Gets all imag elements.
     /// # Example
@@ -163,7 +163,7 @@ pub trait ComplexToRealTransformsOpsBuffered<S, T>: ToRealResult
     /// assert_eq!([2.0, 4.0], result[0..]);
     /// # }
     /// ```
-    fn to_imag_b<B>(self, buffer: &mut B) -> Self::RealResult where B: Buffer<S, T>;
+    fn to_imag_b<B>(self, buffer: &mut B) -> Self::RealResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Gets the phase of all elements in [rad].
     /// # Example
@@ -180,7 +180,7 @@ pub trait ComplexToRealTransformsOpsBuffered<S, T>: ToRealResult
     /// assert_eq!([0.0, 1.5707964, 3.1415927, -1.5707964, 0.7853982], result[0..]);
     /// # }
     /// ```
-    fn phase_b<B>(self, buffer: &mut B) -> Self::RealResult where B: Buffer<S, T>;
+    fn phase_b<B>(self, buffer: &mut B) -> Self::RealResult where B: for<'a> Buffer<'a, S, T>;
 }
 
 
@@ -189,8 +189,11 @@ pub trait ComplexToRealTransformsOpsBuffered<S, T>: ToRealResult
 /// # Failures
 /// All operations in this trait set the arguments `len()` to `0` if the type isn't in
 /// the complex number space.
-pub trait ComplexToRealGetterOps<T>: ToRealResult
-    where T: RealNumber
+pub trait ComplexToRealGetterOps<A, T, N, D>
+    where T: RealNumber,
+          N: NumberSpace,
+          D: Domain,
+          A: GetMetaData<T, N, D>
 {
     /// Copies all real elements into the given vector.
     /// # Example
@@ -206,7 +209,7 @@ pub trait ComplexToRealGetterOps<T>: ToRealResult
     /// assert_eq!([1.0, 3.0], target[..]);
     /// # }
     /// ```
-    fn get_real(&self, destination: &mut Self::RealResult);
+    fn get_real(&self, destination: &mut A);
 
     /// Copies all imag elements into the given vector.
     /// # Example
@@ -222,7 +225,7 @@ pub trait ComplexToRealGetterOps<T>: ToRealResult
     /// assert_eq!([2.0, 4.0], target[..]);
     /// # }
     /// ```
-    fn get_imag(&self, destination: &mut Self::RealResult);
+    fn get_imag(&self, destination: &mut A);
 
     /// Copies the absolute value or magnitude of all vector elements into the given target vector.
     /// # Example
@@ -238,7 +241,7 @@ pub trait ComplexToRealGetterOps<T>: ToRealResult
     /// assert_eq!([5.0, 5.0], target[..]);
     /// # }
     /// ```
-    fn get_magnitude(&self, destination: &mut Self::RealResult);
+    fn get_magnitude(&self, destination: &mut A);
 
     /// Copies the absolute value squared or magnitude squared of all vector elements
     /// into the given target vector.
@@ -259,7 +262,7 @@ pub trait ComplexToRealGetterOps<T>: ToRealResult
     /// assert_eq!([25.0, 25.0], target[..]);
     /// # }
     /// ```
-    fn get_magnitude_squared(&self, destination: &mut Self::RealResult);
+    fn get_magnitude_squared(&self, destination: &mut A);
 
     /// Copies the phase of all elements in [rad] into the given vector.
     /// # Example
@@ -282,19 +285,19 @@ pub trait ComplexToRealGetterOps<T>: ToRealResult
     /// }
     /// # }
     /// ```
-    fn get_phase(&self, destination: &mut Self::RealResult);
+    fn get_phase(&self, destination: &mut A);
 
     /// Gets the real and imaginary parts and stores them in the given vectors.
     /// See also  [`get_phase`](trait.ComplexVectorOps.html#tymethod.get_phase) and
     /// [`get_complex_abs`](trait.ComplexVectorOps.html#tymethod.get_complex_abs) for further
     /// information.
-    fn get_real_imag(&self, real: &mut Self::RealResult, imag: &mut Self::RealResult);
+    fn get_real_imag(&self, real: &mut A, imag: &mut A);
 
     /// Gets the magnitude and phase and stores them in the given vectors.
     /// See also [`get_real`](trait.ComplexVectorOps.html#tymethod.get_real) and
     /// [`get_imag`](trait.ComplexVectorOps.html#tymethod.get_imag) for further
     /// information.
-    fn get_mag_phase(&self, mag: &mut Self::RealResult, phase: &mut Self::RealResult);
+    fn get_mag_phase(&self, mag: &mut A, phase: &mut A);
 }
 
 /// Defines setters to create complex data from real data.
@@ -302,18 +305,21 @@ pub trait ComplexToRealGetterOps<T>: ToRealResult
 /// # Failures
 /// All operations in this trait set `self.len()` to `0` if the type isn't in
 /// the complex number space.
-pub trait ComplexToRealSetterOps<T>: ToRealResult
-    where T: RealNumber
+pub trait ComplexToRealSetterOps<A, T, N, D>
+    where T: RealNumber,
+          N: NumberSpace,
+          D: Domain,
+          A: GetMetaData<T, N, D>
 {
     /// Overrides the `self` vectors data with the real and imaginary data in the given vectors.
     /// `real` and `imag` must have the same size.
-    fn set_real_imag(&mut self, real: &Self::RealResult, imag: &Self::RealResult) -> VoidResult;
+    fn set_real_imag(&mut self, real: &A, imag: &A) -> VoidResult;
 
     /// Overrides the `self` vectors data with the magnitude and phase data in the given vectors.
     /// Note that `self` vector will immediately convert the data into a real and
     /// imaginary representation of the complex numbers which is its default format.
     /// `mag` and `phase` must have the same size.
-    fn set_mag_phase(&mut self, mag: &Self::RealResult, phase: &Self::RealResult) -> VoidResult;
+    fn set_mag_phase(&mut self, mag: &A, phase: &A) -> VoidResult;
 }
 
 macro_rules! assert_complex {
@@ -329,7 +335,7 @@ macro_rules! assert_complex {
 impl<S, T, N, D> ComplexToRealTransformsOps<T> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToRealResult,
           <DspVec<S, T, N, D> as ToRealResult>::RealResult: RededicateForceOps<DspVec<S, T, N, D>>,
-          S: ToSliceMut<T> + Owner,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: ComplexNumberSpace,
           D: Domain
@@ -371,13 +377,13 @@ impl<S, T, N, D> ComplexToRealTransformsOps<T> for DspVec<S, T, N, D>
 impl<S, T, N, D> ComplexToRealTransformsOpsBuffered<S, T> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToRealResult,
           <DspVec<S, T, N, D> as ToRealResult>::RealResult: RededicateForceOps<DspVec<S, T, N, D>>,
-          S: ToSliceMut<T> + Owner,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: ComplexNumberSpace,
           D: Domain
 {
     fn magnitude_b<B>(mut self, buffer: &mut B) -> Self::RealResult
-        where B: Buffer<S, T>
+        where B: for<'a> Buffer<'a, S, T>
     {
         assert_complex!(self);
         self.simd_complex_to_real_operation(buffer,
@@ -390,7 +396,7 @@ impl<S, T, N, D> ComplexToRealTransformsOpsBuffered<S, T> for DspVec<S, T, N, D>
     }
 
     fn magnitude_squared_b<B>(mut self, buffer: &mut B) -> Self::RealResult
-        where B: Buffer<S, T>
+        where B: for<'a> Buffer<'a, S, T>
     {
         assert_complex!(self);
         self.simd_complex_to_real_operation(buffer,
@@ -403,7 +409,7 @@ impl<S, T, N, D> ComplexToRealTransformsOpsBuffered<S, T> for DspVec<S, T, N, D>
     }
 
     fn to_real_b<B>(mut self, buffer: &mut B) -> Self::RealResult
-        where B: Buffer<S, T>
+        where B: for<'a> Buffer<'a, S, T>
     {
         assert_complex!(self);
         self.pure_complex_to_real_operation(buffer, |x, _arg| x.re, (), Complexity::Small);
@@ -412,7 +418,7 @@ impl<S, T, N, D> ComplexToRealTransformsOpsBuffered<S, T> for DspVec<S, T, N, D>
     }
 
     fn to_imag_b<B>(mut self, buffer: &mut B) -> Self::RealResult
-        where B: Buffer<S, T>
+        where B: for<'a> Buffer<'a, S, T>
     {
         assert_complex!(self);
         self.pure_complex_to_real_operation(buffer, |x, _arg| x.im, (), Complexity::Small);
@@ -421,7 +427,7 @@ impl<S, T, N, D> ComplexToRealTransformsOpsBuffered<S, T> for DspVec<S, T, N, D>
     }
 
     fn phase_b<B>(mut self, buffer: &mut B) -> Self::RealResult
-        where B: Buffer<S, T>
+        where B: for<'a> Buffer<'a, S, T>
     {
         assert_complex!(self);
         self.pure_complex_to_real_operation(buffer, |x, _arg| x.arg(), (), Complexity::Small);
@@ -543,38 +549,40 @@ macro_rules! assert_self_complex_and_targets_real {
     }
 }
 
-impl<S, T, N, D> ComplexToRealGetterOps<T> for DspVec<S, T, N, D>
+impl<S, T, N, NR, D, O, DO> ComplexToRealGetterOps<O, T, NR, DO> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToRealResult,
-          <DspVec<S, T, N, D> as ToRealResult>::RealResult:
-            Vector<T>
+          O:
+            Vector<T> + GetMetaData<T, NR, DO>
             + Index<Range<usize>, Output=[T]>
             + IndexMut<Range<usize>>,
           S: ToSlice<T>,
           T: RealNumber,
           N: ComplexNumberSpace,
-          D: Domain {
-    fn get_real(&self, destination: &mut Self::RealResult) {
+          NR: RealNumberSpace,
+          D: Domain,
+          DO: PosEq<D> + Domain {
+    fn get_real(&self, destination: &mut O) {
         assert_self_complex_and_target_real!(self, destination);
         destination.resize(self.len()).expect("Target is real and thus all values are valid");
         self.pure_complex_into_real_target_operation(
             destination, |x,_arg|x.re, (), Complexity::Small);
     }
 
-    fn get_imag(&self, destination: &mut Self::RealResult) {
+    fn get_imag(&self, destination: &mut O) {
         assert_self_complex_and_target_real!(self, destination);
         destination.resize(self.len()).expect("Target is real and thus all values are valid");
         self.pure_complex_into_real_target_operation(
             destination, |x,_arg|x.im, (), Complexity::Small);
     }
 
-    fn get_magnitude(&self, destination: &mut Self::RealResult) {
+    fn get_magnitude(&self, destination: &mut O) {
         assert_self_complex_and_target_real!(self, destination);
         destination.resize(self.len()).expect("Target is real and thus all values are valid");
         self.simd_complex_into_real_target_operation(
             destination, |x|x.complex_abs(), |x|x.norm(), Complexity::Small);
     }
 
-    fn get_magnitude_squared(&self, destination: &mut Self::RealResult) {
+    fn get_magnitude_squared(&self, destination: &mut O) {
         assert_self_complex_and_target_real!(self, destination);
         destination.resize(self.len()).expect("Target is real and thus all values are valid");
         self.simd_complex_into_real_target_operation(
@@ -584,14 +592,14 @@ impl<S, T, N, D> ComplexToRealGetterOps<T> for DspVec<S, T, N, D>
             Complexity::Small);
     }
 
-    fn get_phase(&self, destination: &mut Self::RealResult) {
+    fn get_phase(&self, destination: &mut O) {
         assert_self_complex_and_target_real!(self, destination);
         destination.resize(self.len()).expect("Target is real and thus all values are valid");
         self.pure_complex_into_real_target_operation(
             destination, |x,_arg|x.arg(), (), Complexity::Small)
     }
 
-    fn get_real_imag(&self, real: &mut Self::RealResult, imag: &mut Self::RealResult) {
+    fn get_real_imag(&self, real: &mut O, imag: &mut O) {
         assert_self_complex_and_targets_real!(self, real, imag);
         let data_length = self.len();
         real.resize(data_length / 2).expect(
@@ -610,7 +618,7 @@ impl<S, T, N, D> ComplexToRealGetterOps<T> for DspVec<S, T, N, D>
         }
     }
 
-    fn get_mag_phase(&self, mag: &mut Self::RealResult, phase: &mut Self::RealResult) {
+    fn get_mag_phase(&self, mag: &mut O, phase: &mut O) {
         assert_self_complex_and_targets_real!(self, mag, phase);
         let data_length = self.len();
         mag.resize(data_length / 2).expect(
@@ -631,16 +639,17 @@ impl<S, T, N, D> ComplexToRealGetterOps<T> for DspVec<S, T, N, D>
     }
 }
 
-impl<S, T, N, D> ComplexToRealSetterOps<T> for DspVec<S, T, N, D>
+impl<S, T, N, NR, D, O, DO> ComplexToRealSetterOps<O, T, NR, DO> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToRealResult,
-          <DspVec<S, T, N, D> as ToRealResult>::RealResult:
-            Index<Range<usize>, Output=[T]>
-            + Vector<T>,
-          S: ToSliceMut<T> + Owner + Resize,
+          O: Index<Range<usize>, Output = [T]> + Vector<T> + GetMetaData<T, NR, DO>,
+          S: ToSliceMut<T> + Resize,
           T: RealNumber,
           N: ComplexNumberSpace,
-          D: Domain {
-    fn set_real_imag(&mut self, real: &Self::RealResult, imag: &Self::RealResult) -> VoidResult {
+          NR: RealNumberSpace,
+          D: Domain,
+          DO: PosEq<D> + Domain
+{
+    fn set_real_imag(&mut self, real: &O, imag: &O) -> VoidResult {
         {
             if real.len() != imag.len() {
                 return Err(ErrorReason::InvalidArgumentLength);
@@ -663,7 +672,7 @@ impl<S, T, N, D> ComplexToRealSetterOps<T> for DspVec<S, T, N, D>
         Ok(())
     }
 
-    fn set_mag_phase(&mut self, mag: &Self::RealResult, phase: &Self::RealResult) -> VoidResult {
+    fn set_mag_phase(&mut self, mag: &O, phase: &O) -> VoidResult {
         {
             if mag.len() != phase.len() {
                 return Err(ErrorReason::InvalidArgumentLength);
