@@ -1,11 +1,11 @@
 //! Fundamental math operations
-use {RealNumber, array_to_complex, array_to_complex_mut};
+use {array_to_complex, array_to_complex_mut};
 use multicore_support::*;
 use simd_extensions::*;
 use numbers::*;
 use std::ops::*;
-use super::super::{ErrorReason, VoidResult, Vector,
-                   DspVec, ToSliceMut, MetaData, Domain, NumberSpace, ComplexNumberSpace};
+use super::super::{ErrorReason, VoidResult, Vector, GetMetaData, DspVec, ToSliceMut, MetaData,
+                   Domain, NumberSpace, ComplexNumberSpace, PosEq};
 
 /// An operation which multiplies each vector element with a constant
 pub trait ScaleOps<T>: Sized
@@ -60,7 +60,8 @@ pub trait OffsetOps<T>: Sized
 }
 
 /// Elementary algebra on types: addition, subtraction, multiplication and division
-pub trait ElementaryOps<A> {
+pub trait ElementaryOps<A, T: RealNumber, N: NumberSpace, D: Domain> 
+    where A: GetMetaData<T, N, D> {
     /// Calculates the sum of `self + summand`. It consumes self and returns the result.
     /// # Failures
     /// TransRes may report the following `ErrorReason` members:
@@ -155,7 +156,8 @@ pub trait ElementaryOps<A> {
 }
 
 /// Elementary algebra on types where the argument might contain less data points than `self`.
-pub trait ElementaryWrapAroundOps<A> {
+pub trait ElementaryWrapAroundOps<A, T: RealNumber, N: NumberSpace, D: Domain> 
+    where A: GetMetaData<T, N, D> {
     /// Calculates the sum of `self + summand`. `summand` may be smaller than `self` as long
     /// as `self.len() % summand.len() == 0`. THe result is the same as it would be if
     /// you would repeat `summand` until it has the same length as `self`.
@@ -348,9 +350,9 @@ macro_rules! reject_if {
 macro_rules! assert_meta_data {
     ($self_: ident, $other: ident) => {
          {
-            let delta_ratio = $self_.delta / $other.delta;
+            let delta_ratio = $self_.delta() / $other.delta();
             if $self_.is_complex() != $other.is_complex() ||
-                $self_.domain != $other.domain ||
+                $self_.domain() != $other.domain() ||
                 delta_ratio > T::from(1.1).unwrap() || delta_ratio < T::from(0.9).unwrap() {
                 return Err(ErrorReason::InputMetaDataMustAgree);
             }
@@ -360,7 +362,7 @@ macro_rules! assert_meta_data {
 
 macro_rules! impl_binary_vector_operation {
     (fn $method: ident, $arg_name: ident, $simd_op: ident, $scal_op: ident) => {
-        fn $method(&mut self, $arg_name: &Self) -> VoidResult
+        fn $method<O: Vector<T> + Index<RangeFull, Output=[T]>>(&mut self, $arg_name: &O) -> VoidResult
         {
             {
                 let len = self.len();
@@ -371,7 +373,7 @@ macro_rules! impl_binary_vector_operation {
                 let mut array = self.data.to_slice_mut();
                 let (scalar_left, scalar_right, vectorization_length) =
                     T::Reg::calc_data_alignment_reqs(&array[0..data_length]);
-                let other = &$arg_name.data.to_slice();
+                let other = &$arg_name[..];
                 if vectorization_length > 0 {
                     Chunk::from_src_to_dest(
                         Complexity::Small, &self.multicore_settings,
@@ -402,7 +404,7 @@ macro_rules! impl_binary_vector_operation {
 
 macro_rules! impl_binary_complex_vector_operation {
     (fn $method: ident, $arg_name: ident, $simd_op: ident, $scal_op: ident) => {
-        fn $method(&mut self, $arg_name: &Self) -> VoidResult
+        fn $method<O: Vector<T> + Index<RangeFull, Output=[T]>>(&mut self, $arg_name: &O) -> VoidResult
         {
             {
                 let len = self.len();
@@ -413,7 +415,7 @@ macro_rules! impl_binary_complex_vector_operation {
                 let mut array = self.data.to_slice_mut();
                 let (scalar_left, scalar_right, vectorization_length) =
                     T::Reg::calc_data_alignment_reqs(&array[0..data_length]);
-                let other = &$arg_name.data.to_slice();
+                let other = &$arg_name[..];
                 if vectorization_length > 0 {
                     Chunk::from_src_to_dest(
                         Complexity::Small, &self.multicore_settings,
@@ -457,7 +459,7 @@ macro_rules! impl_binary_complex_vector_operation {
 
 macro_rules! impl_binary_smaller_vector_operation {
     (fn $method: ident, $arg_name: ident, $simd_op: ident, $scal_op: ident) => {
-        fn $method(&mut self, $arg_name: &Self) -> VoidResult
+        fn $method<O: Vector<T> + Index<RangeFull, Output=[T]>>(&mut self, $arg_name: &O) -> VoidResult
         {
             {
                 let len = self.len();
@@ -466,7 +468,7 @@ macro_rules! impl_binary_smaller_vector_operation {
 
                 let data_length = self.len();
                 let mut array = self.data.to_slice_mut();
-                let other = $arg_name.data.to_slice();
+                let other = &$arg_name[..];
                 Chunk::from_src_to_dest(
                     Complexity::Small, &self.multicore_settings,
                     &other, T::Reg::len(),
@@ -487,7 +489,7 @@ macro_rules! impl_binary_smaller_vector_operation {
 
 macro_rules! impl_binary_smaller_complex_vector_ops {
     (fn $method: ident, $arg_name: ident, $simd_op: ident, $scal_op: ident) => {
-        fn $method(&mut self, $arg_name: &Self) -> VoidResult
+        fn $method<O: Vector<T> + Index<RangeFull, Output=[T]>>(&mut self, $arg_name: &O) -> VoidResult
         {
             {
                 let len = self.len();
@@ -496,7 +498,7 @@ macro_rules! impl_binary_smaller_complex_vector_ops {
 
                 let data_length = self.len();
                 let mut array = self.data.to_slice_mut();
-                let other = $arg_name.data.to_slice();
+                let other = &$arg_name[..];
                 Chunk::from_src_to_dest(
                     Complexity::Small, &self.multicore_settings,
                     &other, T::Reg::len(),
@@ -523,6 +525,10 @@ impl<S, T, N, D> DspVec<S, T, N, D>
           N: NumberSpace,
           D: Domain
 {
+    impl_binary_smaller_vector_operation!(fn add_smaller_inter, summand, add, add);
+    impl_binary_smaller_vector_operation!(fn sub_smaller_inter, subtrahend, sub, sub);
+    impl_binary_vector_operation!(fn add_inter, summand, add, add);
+    impl_binary_vector_operation!(fn sub_inter, subtrahend, sub, sub);
     impl_binary_complex_vector_operation!(fn mul_complex, factor, mul_complex, mul);
     impl_binary_smaller_complex_vector_ops!(fn mul_smaller_complex, factor, mul_complex, mul);
     impl_binary_vector_operation!(fn mul_real, factor, mul, mul);
@@ -533,16 +539,24 @@ impl<S, T, N, D> DspVec<S, T, N, D>
     impl_binary_smaller_vector_operation!(fn div_smaller_real, divisor, div, div);
 }
 
-impl<S, T, N, D> ElementaryOps<DspVec<S, T, N, D>> for DspVec<S, T, N, D>
+impl<S, T, N, D, O, NO, DO> ElementaryOps<O, T, NO, DO> for DspVec<S, T, N, D>
     where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
-          D: Domain
+          D: Domain,
+          O: Vector<T> + Index<RangeFull, Output = [T]> + GetMetaData<T, NO, DO>,
+          NO: PosEq<N> + NumberSpace,
+          DO: PosEq<D> + Domain
 {
-    impl_binary_vector_operation!(fn add, summand, add, add);
-    impl_binary_vector_operation!(fn sub, summand, sub, sub);
+    fn add(&mut self, summand: &O) -> VoidResult {
+        self.add_inter(summand)
+    }
 
-    fn mul(&mut self, factor: &Self) -> VoidResult {
+    fn sub(&mut self, subtrahend: &O) -> VoidResult {
+        self.sub_inter(subtrahend)
+    }
+
+    fn mul(&mut self, factor: &O) -> VoidResult {
         let len = self.len();
         reject_if!(self, len != factor.len(), ErrorReason::InputMustHaveTheSameSize);
         assert_meta_data!(self, factor);
@@ -554,7 +568,7 @@ impl<S, T, N, D> ElementaryOps<DspVec<S, T, N, D>> for DspVec<S, T, N, D>
         }
     }
 
-    fn div(&mut self, divisor: &Self) -> VoidResult {
+    fn div(&mut self, divisor: &O) -> VoidResult {
         let len = self.len();
         reject_if!(self, len != divisor.len(), ErrorReason::InputMustHaveTheSameSize);
         assert_meta_data!(self, divisor);
@@ -567,16 +581,23 @@ impl<S, T, N, D> ElementaryOps<DspVec<S, T, N, D>> for DspVec<S, T, N, D>
     }
 }
 
-impl<S, T, N, D> ElementaryWrapAroundOps<DspVec<S, T, N, D>> for DspVec<S, T, N, D>
+impl<S, T, N, D, O, NO, DO> ElementaryWrapAroundOps<O, T, NO, DO> for DspVec<S, T, N, D>
     where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
-          D: Domain
-{
-    impl_binary_smaller_vector_operation!(fn add_smaller, summand, add, add);
-    impl_binary_smaller_vector_operation!(fn sub_smaller, summand, sub, sub);
+          D: Domain,
+          O: Vector<T> + Index<RangeFull, Output = [T]> + GetMetaData<T, NO, DO>,
+          NO: PosEq<N> + NumberSpace,
+          DO: PosEq<D> + Domain {
+    fn add_smaller(&mut self, summand: &O) -> VoidResult {
+        self.add_smaller_inter(summand)
+    }
 
-    fn mul_smaller(&mut self, factor: &Self) -> VoidResult {
+    fn sub_smaller(&mut self, subtrahend: &O) -> VoidResult {
+        self.sub_smaller_inter(subtrahend)
+    }
+
+    fn mul_smaller(&mut self, factor: &O) -> VoidResult {
         let len = self.len();
         reject_if!(self, len % factor.len() != 0, ErrorReason::InvalidArgumentLength);
         assert_meta_data!(self, factor);
@@ -588,7 +609,7 @@ impl<S, T, N, D> ElementaryWrapAroundOps<DspVec<S, T, N, D>> for DspVec<S, T, N,
         }
     }
 
-    fn div_smaller(&mut self, divisor: &Self) -> VoidResult {
+    fn div_smaller(&mut self, divisor: &O) -> VoidResult {
         let len = self.len();
         reject_if!(self, len % divisor.len() != 0, ErrorReason::InvalidArgumentLength);
         assert_meta_data!(self, divisor);
