@@ -1,4 +1,4 @@
-use {RealNumber, array_to_complex, array_to_complex_mut, Zero};
+use {array_to_complex, array_to_complex_mut};
 use multicore_support::{Chunk, MultiCoreSettings, Complexity};
 use numbers::*;
 use std::ops::*;
@@ -7,11 +7,18 @@ use std::cmp;
 use std::result;
 use simd_extensions::*;
 use std::fmt;
+use std;
 
 mod requirements;
 pub use self::requirements::*;
 mod to_from_vec_conversions;
 pub use self::to_from_vec_conversions::*;
+mod support_core;
+pub use self::support_core::*;
+#[cfg(any(feature="std", test))]
+mod support_std;
+#[cfg(any(feature="std", test))]
+pub use self::support_std::*;
 mod vec_impl_and_indexers;
 pub use self::vec_impl_and_indexers::*;
 mod complex;
@@ -20,7 +27,6 @@ mod real;
 pub use self::real::*;
 mod time_freq;
 pub use self::time_freq::*;
-#[cfg(feature="std")]
 pub mod combined_ops;
 mod rededicate_and_relations;
 pub use self::rededicate_and_relations::*;
@@ -49,9 +55,9 @@ pub type ScalarResult<T> = result::Result<T, ErrorReason>;
 #[derive(PartialEq)]
 #[derive(Debug)]
 pub enum DataDomain {
-    /// Time domain, the x-axis is in [s]
+    /// Time domain, the x-axis is in [s].
     Time,
-    /// Frequency domain, the x-axis in [Hz]
+    /// Frequency domain, the x-axis is in [Hz].
     Frequency,
 }
 
@@ -65,7 +71,7 @@ pub trait NumberSpace: fmt::Debug + cmp::PartialEq + Clone {
     /// they may leave this empty.
     fn to_complex(&mut self);
 
-    /// See `RealNumberSpace` for some more details.
+    /// See `to_complex` for more details.
     fn to_real(&mut self);
 }
 
@@ -73,10 +79,10 @@ pub trait NumberSpace: fmt::Debug + cmp::PartialEq + Clone {
 pub trait Domain: fmt::Debug + cmp::PartialEq + Clone {
     fn domain(&self) -> DataDomain;
 
-    /// See `RealNumberSpace` for some more details.
+    /// See `to_complex` for some details.
     fn to_freq(&mut self);
 
-    /// See `RealNumberSpace` for some more details.
+    /// See `to_complex` for some details.
     fn to_time(&mut self);
 }
 
@@ -195,10 +201,26 @@ impl Domain for TimeOrFrequencyData {
 impl TimeDomain for TimeOrFrequencyData {}
 impl FrequencyDomain for TimeOrFrequencyData {}
 
+/// Expresses at compile time that two classes could potentially represent the same number space or domain. 
+pub trait PosEq<O> { }
+impl PosEq<RealData> for RealData { }
+impl PosEq<RealOrComplexData> for RealData { }
+impl PosEq<RealData> for RealOrComplexData { }
+impl PosEq<ComplexData> for RealOrComplexData { }
+impl PosEq<RealOrComplexData> for RealOrComplexData { }
+impl PosEq<ComplexData> for ComplexData { }
+impl PosEq<RealOrComplexData> for ComplexData { }
+
+impl PosEq<TimeData> for TimeData { }
+impl PosEq<TimeOrFrequencyData> for TimeData { }
+impl PosEq<TimeData> for TimeOrFrequencyData { }
+impl PosEq<FrequencyData> for TimeOrFrequencyData { }
+impl PosEq<TimeOrFrequencyData> for TimeOrFrequencyData { }
+impl PosEq<FrequencyData> for FrequencyData { }
+impl PosEq<TimeOrFrequencyData> for FrequencyData { }
+    
 /// A 1xN (one times N elements) or Nx1 data vector as used for most digital signal processing
-/// (DSP) operations. All data vector operations consume the vector they operate on and return a
-/// new vector. A consumed vector
-/// must not be accessed again.
+/// (DSP) operations.
 ///
 /// Vectors come in different flavors:
 ///
@@ -258,75 +280,16 @@ impl<S, T, N, D> fmt::Debug for DspVec<S, T, N, D>
     }
 }
 
-/// A vector with real numbers in time domain.
-pub type RealTimeVec<S, T> = DspVec<S, T, RealData, TimeData>;
-/// A vector with real numbers in frequency domain.
-pub type RealFreqVec<S, T> = DspVec<S, T, RealData, FrequencyData>;
-/// A vector with complex numbers in time domain.
-pub type ComplexTimeVec<S, T> = DspVec<S, T, ComplexData, TimeData>;
-/// A vector with complex numbers in frequency domain.
-pub type ComplexFreqVec<S, T> = DspVec<S, T, ComplexData, FrequencyData>;
-/// A vector with no information about number space or domain at compile time.
-pub type GenDspVec<S, T> = DspVec<S, T, RealOrComplexData, TimeOrFrequencyData>;
-
-/// A vector with real numbers in time domain.
-#[cfg(feature="std")]
-pub type RealTimeVec32 = DspVec<Vec<f32>, f32, RealData, TimeData>;
-/// A vector with real numbers in frequency domain.
-#[cfg(feature="std")]
-pub type RealFreqVec32 = DspVec<Vec<f32>, f32, RealData, FrequencyData>;
-/// A vector with complex numbers in time domain.
-#[cfg(feature="std")]
-pub type ComplexTimeVec32 = DspVec<Vec<f32>, f32, ComplexData, TimeData>;
-/// A vector with complex numbers in frequency domain.
-#[cfg(feature="std")]
-pub type ComplexFreqVec32 = DspVec<Vec<f32>, f32, ComplexData, FrequencyData>;
-/// A vector with no information about number space or domain at compile time.
-#[cfg(feature="std")]
-pub type GenDspVec32 = DspVec<Vec<f32>, f32, RealOrComplexData, TimeOrFrequencyData>;
-
-/// A vector with real numbers in time domain.
-pub type RealTimeVecSlice32<'a> = DspVec<&'a [f32], f32, RealData, TimeData>;
-/// A vector with real numbers in frequency domain.
-pub type RealFreqVecSlice32<'a> = DspVec<&'a [f32], f32, RealData, FrequencyData>;
-/// A vector with complex numbers in time domain.
-pub type ComplexTimeVecSlice32<'a> = DspVec<&'a [f32], f32, ComplexData, TimeData>;
-/// A vector with complex numbers in frequency domain.
-pub type ComplexFreqVecSlice32<'a> = DspVec<&'a [f32], f32, ComplexData, FrequencyData>;
-/// A vector with no information about number space or domain at compile time.
-pub type GenDspVecSlice32<'a> = DspVec<&'a [f32], f32, RealOrComplexData, TimeOrFrequencyData>;
-
-/// A vector with real numbers in time domain.
-#[cfg(feature="std")]
-pub type RealTimeVec64 = DspVec<Vec<f64>, f64, RealData, TimeData>;
-/// A vector with real numbers in frequency domain.
-#[cfg(feature="std")]
-pub type RealFreqVec64 = DspVec<Vec<f64>, f64, RealData, FrequencyData>;
-/// A vector with complex numbers in time domain.
-#[cfg(feature="std")]
-pub type ComplexTimeVec64 = DspVec<Vec<f64>, f64, ComplexData, TimeData>;
-/// A vector with complex numbers in frequency domain.
-#[cfg(feature="std")]
-pub type ComplexFreqVec64 = DspVec<Vec<f64>, f64, ComplexData, FrequencyData>;
-/// A vector with no information about number space or domain at compile time.
-#[cfg(feature="std")]
-pub type GenDspVec64 = DspVec<Vec<f64>, f64, RealOrComplexData, TimeOrFrequencyData>;
-
-/// A vector with real numbers in time domain.
-pub type RealTimeVecSlice64<'a> = DspVec<&'a [f64], f64, RealData, TimeData>;
-/// A vector with real numbers in frequency domain.
-pub type RealFreqVecSlice64<'a> = DspVec<&'a [f64], f64, RealData, FrequencyData>;
-/// A vector with complex numbers in time domain.
-pub type ComplexTimeVecSlice64<'a> = DspVec<&'a [f64], f64, ComplexData, TimeData>;
-/// A vector with complex numbers in frequency domain.
-pub type ComplexFreqVecSlice64<'a> = DspVec<&'a [f64], f64, ComplexData, FrequencyData>;
-/// A vector with no information about number space or domain at compile time.
-pub type GenDspVecSlice64<'a> = DspVec<&'a [f64], f64, RealOrComplexData, TimeOrFrequencyData>;
-
+/// Rounds a length so that it always devides by the length of a SIMD
+/// register. This function assumes that `Reg32::len() > Reg64::len()`.
 fn round_len(len: usize) -> usize {
     ((len + Reg32::len() - 1) / Reg32::len()) * Reg32::len()
 }
 
+/// Swaps the halves of two arrays. The `forward` paremeter
+/// specifies what should happen then the `data.len()` is odd.
+/// The function should always produce the same results as `fft_shift`
+/// and `ifft_shift` in GNU Octave.
 fn swap_array_halves<T>(data: &mut [T], forward: bool)
     where T: Copy
 {
@@ -337,9 +300,7 @@ fn swap_array_halves<T>(data: &mut [T], forward: bool)
             mem::swap(lo, up);
         }
     } else {
-        let step =
-            if forward { len / 2 }
-            else { len / 2 + 1 };
+        let step = if forward { len / 2 } else { len / 2 + 1 };
         let mut temp = data[0];
         let mut pos = step;
         for _ in 0..len {
@@ -350,28 +311,29 @@ fn swap_array_halves<T>(data: &mut [T], forward: bool)
     }
 }
 
-impl<S, T, N, D> DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + ToDspVector<T>,
-          T: RealNumber,
-          N: NumberSpace,
-          D: Domain
+/// Creates an interleaved array slice from a complex array.
+fn complex_to_array<T>(complex: &[Complex<T>]) -> &[T]
+    where T: RealNumber
 {
-    fn swap_data<N1, D1>(&mut self, other: &mut DspVec<S, T, N1, D1>)
-        where N1: NumberSpace, D1: Domain {
-            let self_len = self.len();
-            let other_len = other.len();
-            mem::swap(&mut self.data, &mut other.data);
-            self.resize(other_len)
-                .expect("Resize after swap should succeed");
-            other.resize(self_len)
-                .expect("Resize after swap should succeed");
+    use std::slice;
+    use std::mem;
+    unsafe {
+        let len = complex.len();
+        let trans: &[T] = mem::transmute(complex);
+        slice::from_raw_parts(&trans[0] as *const T, len * 2)
     }
+}
 
-    fn take_ownership(&mut self, empty: S) -> Self {
-        let meta_data = self.get_meta_data();
-        let mut other = empty.to_dsp_vec(&meta_data);
-        self.swap_data(&mut other);
-        other
+/// Creates an interleaved array slice from a complex array.
+fn complex_to_array_mut<T>(complex: &mut [Complex<T>]) -> &mut [T]
+    where T: RealNumber
+{
+    use std::slice;
+    use std::mem;
+    unsafe {
+        let len = complex.len();
+        let mut trans: &mut [T] = mem::transmute(complex);
+        slice::from_raw_parts_mut(&mut trans[0] as *mut T, len * 2)
     }
 }
 
@@ -381,6 +343,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
           N: NumberSpace,
           D: Domain
 {
+	/// Executes a real function.
     #[inline]
     fn pure_real_operation<A, F>(&mut self, op: F, argument: A, complexity: Complexity)
         where A: Sync + Copy + Send,
@@ -402,6 +365,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         }
     }
 
+	/// Executes a complex function.
     #[inline]
     fn pure_complex_operation<A, F>(&mut self, op: F, argument: A, complexity: Complexity)
         where A: Sync + Copy + Send,
@@ -424,6 +388,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         }
     }
 
+	/// Executes a real function with SIMD optimization.
     #[inline]
     fn simd_real_operation<A, F, G>(&mut self,
                                     simd_op: F,
@@ -461,6 +426,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         }
     }
 
+	/// Executes a complex function with SIMD optimization.
     #[inline]
     fn simd_complex_operation<A, F, G>(&mut self,
                                        simd_op: F,
@@ -504,6 +470,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         }
     }
 
+	/// Executes a function which converts a complex array into a real array.
     #[inline]
     fn pure_complex_to_real_operation<A, F, B>(&mut self,
                                                buffer: &mut B,
@@ -512,11 +479,11 @@ impl<S, T, N, D> DspVec<S, T, N, D>
                                                complexity: Complexity)
         where A: Sync + Copy + Send,
               F: Fn(Complex<T>, A) -> T + 'static + Sync,
-              B: Buffer<S, T>
+              B: for<'a> Buffer<'a, S, T>
     {
         {
             let data_length = self.len();
-            let mut result = buffer.get(data_length / 2);
+            let mut result = buffer.borrow(data_length / 2);
             {
                 let array = self.data.to_slice_mut();
                 let mut temp = result.to_slice_mut();
@@ -536,11 +503,11 @@ impl<S, T, N, D> DspVec<S, T, N, D>
                 });
                 self.valid_len = data_length / 2;
             }
-            mem::swap(&mut self.data, &mut result);
-            buffer.free(result);
+            result.trade(&mut self.data);
         }
     }
 
+	/// Executes a function which converts a complex array into a real array. Conversion is applied in-place.
     #[inline]
     fn pure_complex_to_real_operation_inplace<A, F>(&mut self, op: F, argument: A)
         where A: Sync + Copy + Send,
@@ -558,6 +525,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         }
     }
 
+	/// Executes a function which converts a complex array into a real array.
     #[inline]
     fn simd_complex_to_real_operation<A, F, G, B>(&mut self,
                                                   buffer: &mut B,
@@ -568,10 +536,10 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         where A: Sync + Copy + Send,
               F: Fn(T::Reg, A) -> T::Reg + 'static + Sync,
               G: Fn(Complex<T>, A) -> T + 'static + Sync,
-              B: Buffer<S, T>
+              B: for<'a> Buffer<'a, S, T>
     {
         let data_length = self.len();
-        let mut result = buffer.get(data_length / 2);
+        let mut result = buffer.borrow(data_length / 2);
         {
             let array = self.data.to_slice_mut();
             let mut temp = result.to_slice_mut();
@@ -612,13 +580,12 @@ impl<S, T, N, D> DspVec<S, T, N, D>
 
             self.valid_len /= 2;
         }
-        mem::swap(&mut self.data, &mut result);
-        buffer.free(result);
+        result.trade(&mut self.data);
     }
 
+	/// Forwards calls to `swap_array_halves`.
     #[inline]
-    fn swap_halves_priv(&mut self, forward: bool)
-    {
+    fn swap_halves_priv(&mut self, forward: bool) {
         let len = self.len();
         if len == 0 {
             return;
@@ -634,6 +601,7 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         }
     }
 
+	/// Multiplies the vector with a window function.
     #[inline]
     fn multiply_window_priv<TT, CMut, FA, F>(&mut self,
                                              is_symmetric: bool,
@@ -701,6 +669,74 @@ impl<S, T, N, D> DspVec<S, T, N, D>
     }
 }
 
+/// Buffer borrow type for `NoTradeBufferBurrow`.
+pub struct NoTradeBufferBurrow<'a, T: RealNumber + 'a> {
+    data: &'a mut [T],
+}
+
+impl<'a, T: RealNumber> Deref for NoTradeBufferBurrow<'a, T> {
+    type Target = [T];
+
+    fn deref(&self) -> &[T] {
+        self.data
+    }
+}
+
+impl<'a, T: RealNumber> DerefMut for NoTradeBufferBurrow<'a, T> {
+    fn deref_mut(&mut self) -> &mut [T] {
+        self.data
+    }
+}
+
+impl<'a, S: ToSliceMut<T>, T: RealNumber> BufferBorrow<S, T> for NoTradeBufferBurrow<'a, T> {
+    fn trade(self, _: &mut S) {
+        // No trade
+    }
+}
+
+/// For internal use only. A buffer which doesn't implement the `swap` routine. Swapping is a no-op in this
+/// implementation. This can be useful in cases where an implementation will do the swap step on its own.
+struct NoTradeBuffer<S, T>
+    where S: ToSliceMut<T>,
+          T: RealNumber
+{
+    data: S,
+    data_type: std::marker::PhantomData<T>,
+}
+
+impl<S, T> NoTradeBuffer<S, T>
+    where S: ToSliceMut<T>,
+          T: RealNumber
+{
+    /// Creates a new buffer from a storage type. The buffer will internally hold
+    /// its storage for it's complete life time.
+    pub fn new(storage: S) -> NoTradeBuffer<S, T> {
+        NoTradeBuffer {
+            data: storage,
+            data_type: std::marker::PhantomData,
+        }
+    }
+}
+
+impl<'a, S, T> Buffer<'a, S, T> for NoTradeBuffer<S, T>
+    where S: ToSliceMut<T>,
+          T: RealNumber + 'a
+{
+    type Borrow = NoTradeBufferBurrow<'a, T>;
+
+    fn borrow(&'a mut self, len: usize) -> Self::Borrow {
+        if self.data.len() < len {
+            panic!("NoTradeBuffer: Out of memory");
+        }
+
+        NoTradeBufferBurrow { data: &mut self.data.to_slice_mut()[0..len] }
+    }
+
+    fn alloc_len(&self) -> usize {
+        self.data.len()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::swap_array_halves;
