@@ -1,15 +1,16 @@
-use {RealNumber, array_to_complex, array_to_complex_mut};
+use {array_to_complex, array_to_complex_mut};
 use conv_types::*;
 use numbers::*;
+use inline_vector::InlineVector;
 use multicore_support::*;
 use rustfft::FFT;
 use std::mem;
 use std::slice;
 use gpu_support::GpuSupport;
-use super::super::{VoidResult, ToSliceMut, MetaData,
-                   DspVec, NumberSpace, TimeDomain, FrequencyDomain, DataDomain, Vector,
-                   ComplexNumberSpace, Buffer, ErrorReason, TimeToFrequencyDomainOperations, ToComplexVector,
-                   FromVector, PaddingOption, ResizeOps, InsertZerosOps};
+use super::super::{VoidResult, ToSliceMut, MetaData, DspVec, NumberSpace, TimeDomain,
+                   FrequencyDomain, DataDomain, Vector, ComplexNumberSpace, Buffer, BufferBorrow,
+                   ErrorReason, TimeToFrequencyDomainOperations, ToComplexVector, FromVector,
+                   PaddingOption, ResizeOps, InsertZerosOps, PosEq, Domain};
 
 /// Provides a convolution operations.
 pub trait Convolution<'a, S, T, C: 'a>
@@ -30,13 +31,15 @@ pub trait Convolution<'a, S, T, C: 'a>
     ///    `impulse_response` is in complex number space.
     /// 2. `VectorMustBeInTimeDomain`: if `self` is in frequency domain.
     fn convolve<B>(&mut self, buffer: &mut B, impulse_response: C, ratio: T, len: usize)
-        where B: Buffer<S, T>;
+        where B: for<'b> Buffer<'b, S, T>;
 }
 
 /// Provides a convolution operation for types which at some point are slice based.
-pub trait ConvolutionOps<S, T, A>
+pub trait ConvolutionOps<A, S, T, N, D>
     where S: ToSliceMut<T>,
-          T: RealNumber
+          T: RealNumber,
+          N: NumberSpace,
+          D: Domain
 {
     /// Convolves `self` with the convolution function `impulse_response`.
     /// For performance it's recommended
@@ -52,8 +55,8 @@ pub trait ConvolutionOps<S, T, A>
     /// 2. `VectorMetaDataMustAgree`: in case `self` and `impulse_response`
     ///    are not in the same number space and same domain.
     /// 3. `InvalidArgumentLength`: if `self.points() < impulse_response.points()`.
-    fn convolve_vector<B>(&mut self, buffer: &mut B, impulse_response: &A) -> VoidResult
-        where B: Buffer<S, T>; // TODO: Consider to rename this function with 0.5
+    fn convolve_signal<B>(&mut self, buffer: &mut B, impulse_response: &A) -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 /// Provides a frequency response multiplication operations.
@@ -100,14 +103,14 @@ impl<'a, S, T, N, D> Convolution<'a, S, T, &'a RealImpulseResponse<T>> for DspVe
           T: RealNumber,
           N: NumberSpace,
           D: TimeDomain,
-          DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone
+          DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone + ConvolutionOps<DspVec<InlineVector<T>, T, N, D>, S, T, N, D>
 {
     fn convolve<B>(&mut self,
                    buffer: &mut B,
                    function: &RealImpulseResponse<T>,
                    ratio: T,
                    len: usize)
-        where B: Buffer<S, T>
+        where B: for<'b> Buffer<'b, S, T>
     {
         assert_time!(self);
         if !self.is_complex() {
@@ -121,27 +124,30 @@ impl<'a, S, T, N, D> Convolution<'a, S, T, &'a RealImpulseResponse<T>> for DspVe
                     .expect("Converting ratio to usize failed, is the interpolation factor \
                              perhaps really huge?");
                 let points = (2 * len + 1) * ratio;
-                let mut imp_resp = DspVec {
-                    data: buffer.construct_new(points),
-                    delta: self.delta(),
-                    domain: self.domain.clone(),
-                    number_space: self.number_space.clone(),
-                    valid_len: self.valid_len,
-                    multicore_settings: MultiCoreSettings::default(),
-                };
+                if points <= InlineVector::<T>::max_capacity() {
+                    let mut imp_resp = DspVec {
+                        data: InlineVector::of_size(T::zero(), points),
+                        delta: self.delta(),
+                        domain: self.domain.clone(),
+                        number_space: self.number_space.clone(),
+                        valid_len: points,
+                        multicore_settings: MultiCoreSettings::default(),
+                    };
 
-                let mut i = 0;
-                let mut j = -(T::from(len).unwrap());
-                while i < imp_resp.len() {
-                    let value = function.calc(j * ratio_inv);
-                    imp_resp[i] = value;
-                    i += 1;
-                    j = j + T::one();
-                }
+                    let mut i = 0;
+                    let mut j = -(T::from(len).unwrap());
+                    while i < imp_resp.len() {
+                        let value = function.calc(j * ratio_inv);
+                        imp_resp[i] = value;
+                        i += 1;
+                        j = j + T::one();
+                    }
 
-                self.convolve_vector(buffer, &imp_resp)
-                    .expect("Meta data should agree since we constructed the argument from this \
-                             vector");
+                    self.convolve_signal(buffer, &imp_resp)
+                        .expect("Meta data should agree since we constructed the argument from this \
+                                 vector");
+                    return;
+                 }
             }
 
             self.convolve_function_priv(buffer,
@@ -161,27 +167,30 @@ impl<'a, S, T, N, D> Convolution<'a, S, T, &'a RealImpulseResponse<T>> for DspVe
                     .expect("Converting ratio to usize failed, is the interpolation factor \
                              perhaps really huge?");
                 let points = (2 * len + 1) * ratio;
-                let mut imp_resp = DspVec {
-                    data: buffer.construct_new(2 * points),
-                    delta: self.delta(),
-                    domain: self.domain.clone(),
-                    number_space: self.number_space.clone(),
-                    valid_len: 2 * points,
-                    multicore_settings: MultiCoreSettings::default(),
-                };
+                if 2 * points <= InlineVector::<T>::max_capacity() {
+                    let mut imp_resp = DspVec {
+                        data: InlineVector::of_size(T::zero(), 2 * points),
+                        delta: self.delta(),
+                        domain: self.domain.clone(),
+                        number_space: self.number_space.clone(),
+                        valid_len: 2 * points,
+                        multicore_settings: MultiCoreSettings::default(),
+                    };
 
-                let mut i = 0;
-                let mut j = -(T::from(len).unwrap());
-                while i < imp_resp.len() {
-                    let value = function.calc(j * ratio_inv);
-                    imp_resp[i] = value;
-                    i += 2;
-                    j = j + T::one();
+                    let mut i = 0;
+                    let mut j = -(T::from(len).unwrap());
+                    while i < imp_resp.len() {
+                        let value = function.calc(j * ratio_inv);
+                        imp_resp[i] = value;
+                        i += 2;
+                        j = j + T::one();
+                    }
+
+                    self.convolve_signal(buffer, &imp_resp)
+                        .expect("Meta data should agree since we constructed the argument from this \
+                                 vector");
+                    return;
                 }
-
-                self.convolve_vector(buffer, &imp_resp)
-                    .expect("Meta data should agree since we constructed the argument from this \
-                             vector");
             }
 
             self.convolve_function_priv(buffer,
@@ -199,14 +208,14 @@ impl<'a, S, T, N, D> Convolution<'a, S, T, &'a ComplexImpulseResponse<T>> for Ds
           T: RealNumber,
           N: ComplexNumberSpace,
           D: TimeDomain,
-          DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone
+          DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone + ConvolutionOps<DspVec<InlineVector<T>, T, N, D>, S, T, N, D>
 {
     fn convolve<B>(&mut self,
                    buffer: &mut B,
                    function: &ComplexImpulseResponse<T>,
                    ratio: T,
                    len: usize)
-        where B: Buffer<S, T>
+        where B: for<'b> Buffer<'b, S, T>
     {
         assert_complex!(self);
         assert_time!(self);
@@ -221,29 +230,32 @@ impl<'a, S, T, N, D> Convolution<'a, S, T, &'a ComplexImpulseResponse<T>> for Ds
                 .expect("Converting ratio to usize failed, is the interpolation factor perhaps \
                          really huge?");
             let points = (2 * len + 1) * ratio;
-            let mut imp_resp = DspVec {
-                data: buffer.construct_new(2 * points),
-                delta: self.delta(),
-                domain: self.domain.clone(),
-                number_space: self.number_space.clone(),
-                valid_len: self.valid_len,
-                multicore_settings: MultiCoreSettings::default(),
-            };
+            if 2 * points <= InlineVector::<T>::max_capacity() {
+                let mut imp_resp = DspVec {
+                    data: InlineVector::of_size(T::zero(), 2 * points),
+                    delta: self.delta(),
+                    domain: self.domain.clone(),
+                    number_space: self.number_space.clone(),
+                    valid_len: self.valid_len,
+                    multicore_settings: MultiCoreSettings::default(),
+                };
 
-            let mut i = 0;
-            let mut j = -T::from(len).unwrap();
-            while i < imp_resp.len() {
-                let value = function.calc(j * ratio_inv);
-                imp_resp[i] = value.re;
-                i += 2;
-                imp_resp[i] = value.im;
-                i += 1;
-                j = j + T::one();
-            }
+                let mut i = 0;
+                let mut j = -T::from(len).unwrap();
+                while i < imp_resp.len() {
+                    let value = function.calc(j * ratio_inv);
+                    imp_resp[i] = value.re;
+                    i += 2;
+                    imp_resp[i] = value.im;
+                    i += 1;
+                    j = j + T::one();
+                }
 
-            self.convolve_vector(buffer, &imp_resp)
-                .expect("Meta data should agree since we constructed the argument from this \
-                         vector");
+                self.convolve_signal(buffer, &imp_resp)
+                    .expect("Meta data should agree since we constructed the argument from this \
+                             vector");
+                return;
+             }
         }
 
         self.convolve_function_priv(buffer,
@@ -260,7 +272,7 @@ macro_rules! assert_meta_data {
          {
             let delta_ratio = $self_.delta / $other.delta;
             if $self_.is_complex() != $other.is_complex() ||
-                $self_.domain != $other.domain ||
+                $self_.domain() != $other.domain() ||
                 delta_ratio > T::from(1.1).unwrap() || delta_ratio < T::from(0.9).unwrap() {
                 return Err(ErrorReason::InputMetaDataMustAgree);
             }
@@ -271,7 +283,7 @@ macro_rules! assert_meta_data {
 fn next_power_of_two(value: usize) -> usize {
     let mut count = 0;
     let mut n = value;
-    if n != 0 && (n & (n-1)) == 0 {
+    if n != 0 && (n & (n - 1)) == 0 {
         return n;
     }
     while n != 0 {
@@ -307,8 +319,15 @@ impl<S, T, N, D> DspVec<S, T, N, D>
           D: TimeDomain,
           DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone
 {
-    fn overlap_discard<B>(&mut self, buffer: &mut B, impulse_response: &Self, fft_len: usize) -> VoidResult
-        where B: Buffer<S, T>
+    fn overlap_discard<B, SO, NO, DO>(&mut self,
+                              buffer: &mut B,
+                              impulse_response: &DspVec<SO, T, NO, DO>,
+                              fft_len: usize)
+                              -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>,
+              SO: ToSliceMut<T>,
+              NO: NumberSpace,
+              DO: Domain
     {
         if !self.is_complex() {
             return Err(ErrorReason::InputMustBeComplex);
@@ -321,16 +340,21 @@ impl<S, T, N, D> DspVec<S, T, N, D>
         let x_len = self.points();
         let overlap = imp_len - 1;
         let min_fft_len = next_power_of_two(4 * overlap);
-        let fft_len = if fft_len > min_fft_len { fft_len } else { min_fft_len };
+        let fft_len = if fft_len > min_fft_len {
+            fft_len
+        } else {
+            min_fft_len
+        };
         let mut fft = FFT::new(fft_len, false);
         let mut ifft = FFT::new(fft_len, true);
         let step_size = fft_len - overlap;
-        let h_time_padded_len = 2*fft_len;
-        let h_freq_len = 2*fft_len;
-        let x_freq_len = 2*fft_len;
-        let tmp_len = 2*fft_len;
+        let h_time_padded_len = 2 * fft_len;
+        let h_freq_len = 2 * fft_len;
+        let x_freq_len = 2 * fft_len;
+        let tmp_len = 2 * fft_len;
         let remainder_len = x_len - x_len % fft_len;
-        let mut array = buffer.get(h_time_padded_len + h_freq_len + x_freq_len + tmp_len + remainder_len);
+        let mut array =
+            buffer.borrow(h_time_padded_len + h_freq_len + x_freq_len + tmp_len + remainder_len);
         {
             let mut array = array.to_slice_mut();
             let (mut h_freq, mut array) = array.split_at_mut(h_freq_len);
@@ -339,58 +363,65 @@ impl<S, T, N, D> DspVec<S, T, N, D>
             let (mut h_time_padded, array) = array.split_at_mut(h_time_padded_len);
             let mut end = array;
             let h_time = h_time.data.to_slice();
-            (&mut h_time_padded[0..2*imp_len]).copy_from_slice(&h_time[0..2*imp_len]);
+            (&mut h_time_padded[0..2 * imp_len]).copy_from_slice(&h_time[0..2 * imp_len]);
+
             let mut h_time_padded = (&mut h_time_padded[..]).to_complex_time_vec();
             h_time_padded.resize(2 * imp_len).expect("Shrinking a vector should always succeed");
-            h_time_padded.zero_pad(fft_len, PaddingOption::End).expect("zero padding should always have enough space");
+            h_time_padded.zero_pad(fft_len, PaddingOption::End)
+                .expect("zero padding should always have enough space");
             let (h_time_padded, _) = h_time_padded.get();
 
             let x_time = self.data.to_slice_mut();
             let h_time: &[Complex<T>] = array_to_complex(&h_time[..]);
-            let h_time_padded: &[Complex<T>] = array_to_complex(&h_time_padded[0..2*fft_len]);
-            let x_time: &mut [Complex<T>] = array_to_complex_mut(&mut x_time[0..2*x_len]);
-            let h_freq: &mut [Complex<T>] = array_to_complex_mut(&mut h_freq[0..2*fft_len]);
-            let mut x_freq: &mut [Complex<T>] = array_to_complex_mut(&mut x_freq[0..2*fft_len]);
-            let mut tmp: &mut [Complex<T>] = array_to_complex_mut(&mut tmp[0..2*fft_len]);
+            let h_time_padded: &[Complex<T>] = array_to_complex(&h_time_padded[0..2 * fft_len]);
+            let x_time: &mut [Complex<T>] = array_to_complex_mut(&mut x_time[0..2 * x_len]);
+            let h_freq: &mut [Complex<T>] = array_to_complex_mut(&mut h_freq[0..2 * fft_len]);
+            let mut x_freq: &mut [Complex<T>] = array_to_complex_mut(&mut x_freq[0..2 * fft_len]);
+            let mut tmp: &mut [Complex<T>] = array_to_complex_mut(&mut tmp[0..2 * fft_len]);
             let end: &mut [Complex<T>] = array_to_complex_mut(&mut end[0..remainder_len]);
             fft.process(h_time_padded, h_freq);
-
             let mut position = 0;
             {
                 // (1) Scalar convolution of the beginning
-                for num in &mut tmp[0..imp_len/2] {
-                    *num = Self::convolve_iteration(x_time, h_time, position, ((imp_len + 1) / 2) as isize, imp_len);
+                for num in &mut tmp[0..imp_len / 2] {
+                    *num = Self::convolve_iteration(x_time,
+                                                    h_time,
+                                                    position,
+                                                    ((imp_len + 1) / 2) as isize,
+                                                    imp_len);
                     position += 1;
                 }
-
                 // (2) Scalar convolution of the end/tail
                 position = (x_time.len() - end.len()) as isize;
-                for num in &mut end[0..remainder_len/2] {
-                    *num = Self::convolve_iteration(x_time, h_time, position, ((imp_len + 1) / 2) as isize, imp_len);
+                for num in &mut end[0..remainder_len / 2] {
+                    *num = Self::convolve_iteration(x_time,
+                                                    h_time,
+                                                    position,
+                                                    ((imp_len + 1) / 2) as isize,
+                                                    imp_len);
                     position += 1;
                 }
             }
+
             let mut position = 0;
             if T::has_gpu_support() {
                 // GPU implementation of overlap_discard `thinks` in interleaved arrays
                 // while this implementation `thinks` in complex arrays. Therefore a factor
                 // of 2 needs to be taken into account in the method call.
-                position = T::overlap_discard(
-                    array_to_real_mut(x_time),
-                    array_to_real_mut(tmp),
-                    array_to_real_mut(x_freq),
-                    array_to_real(h_freq),
-                    2 * imp_len,
-                    2 * step_size) / 2;
-            }
-            else {
+                position = T::overlap_discard(array_to_real_mut(x_time),
+                                              array_to_real_mut(tmp),
+                                              array_to_real_mut(x_freq),
+                                              array_to_real(h_freq),
+                                              2 * imp_len,
+                                              2 * step_size) / 2;
+            } else {
                 let scaling = T::from(fft_len).unwrap();
                 {
-                    let range = position .. fft_len+position;
+                    let range = position..fft_len + position;
                     // (3) The first iteration is different since it copies over the results which have been calculated for the beginning
                     fft.process(&x_time[range], x_freq);
                     // Copy over the results of the scalar convolution (1)
-                    (&mut x_time[0..imp_len/2]).copy_from_slice(&tmp[0..imp_len/2]);
+                    (&mut x_time[0..imp_len / 2]).copy_from_slice(&tmp[0..imp_len / 2]);
                     for (n, v) in (&mut x_freq[..]).iter_mut().zip(h_freq.iter()) {
                         *n = *n * *v / scaling;
                     }
@@ -398,11 +429,12 @@ impl<S, T, N, D> DspVec<S, T, N, D>
                     position += step_size;
                 }
 
-                while position+fft_len < x_len {
-                    let range = position .. fft_len+position;
+                while position + fft_len < x_len {
+                    let range = position..fft_len + position;
                     fft.process(&x_time[range], x_freq);
                     // (4) Same as (3) except that the results of the previous iteration gets copied over
-                    (&mut x_time[position-step_size+imp_len/2 .. position+imp_len/2]).copy_from_slice(&tmp[imp_len-1..fft_len]);
+                    (&mut x_time[position - step_size + imp_len / 2..position + imp_len / 2])
+                        .copy_from_slice(&tmp[imp_len - 1..fft_len]);
                     for (n, v) in (&mut x_freq[..]).iter_mut().zip(h_freq.iter()) {
                         *n = *n * *v / scaling;
                     }
@@ -412,26 +444,32 @@ impl<S, T, N, D> DspVec<S, T, N, D>
             }
 
             // (5) now store the result of the last iteration
-            (&mut x_time[position-step_size+imp_len/2 .. position+imp_len/2]).copy_from_slice(&tmp[imp_len-1..fft_len]);
+            (&mut x_time[position - step_size + imp_len / 2..position + imp_len / 2])
+                .copy_from_slice(&tmp[imp_len - 1..fft_len]);
 
             // (6) Copy over the end result which was calculated at the beginning (2)
-            (&mut x_time[x_len - remainder_len / 2 .. ]).copy_from_slice(&end[..]);
+            (&mut x_time[x_len - remainder_len / 2..]).copy_from_slice(&end[..]);
         }
-        buffer.free(array);
-
         Ok(())
     }
 }
 
-impl<S, T, N, D> ConvolutionOps<S, T, DspVec<S, T, N, D>> for DspVec<S, T, N, D>
+impl<S, SO, T, N, D, NO, DO> ConvolutionOps<DspVec<SO, T, NO, DO>, S, T, NO, DO> for DspVec<S, T, N, D>
     where S: ToSliceMut<T>,
+          SO: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: TimeDomain,
-          DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone
+          DspVec<S, T, N, D>: TimeToFrequencyDomainOperations<S, T> + Clone,
+          DspVec<SO, T, N, D>: TimeToFrequencyDomainOperations<SO, T> + Clone,
+          NO: PosEq<N> + NumberSpace,
+          DO: TimeDomain
 {
-    fn convolve_vector<B>(&mut self, buffer: &mut B, impulse_response: &Self) -> VoidResult
-        where B: Buffer<S, T>
+    fn convolve_signal<B>(&mut self,
+                          buffer: &mut B,
+                          impulse_response: &DspVec<SO, T, NO, DO>)
+                          -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>
     {
         assert_meta_data!(self, impulse_response);
         if self.domain() != DataDomain::Time {
@@ -447,10 +485,8 @@ impl<S, T, N, D> ConvolutionOps<S, T, DspVec<S, T, N, D>> for DspVec<S, T, N, D>
         // For the SIMD operation we need to clone `vector` several
         // times and this only is worthwhile if `vector.len() << self.len()`
         // where `<<` means "significant smaller".
-        if self.len() > 1000
-           && impulse_response.len() <= 202
-           && impulse_response.len() > 11 {
-            self.convolve_vector_simd(buffer, impulse_response);
+        if self.len() > 1000 && impulse_response.len() <= 202 && impulse_response.len() > 11 {
+            self.convolve_signal_simd(buffer, impulse_response);
             return Ok(());
         }
 
@@ -458,24 +494,21 @@ impl<S, T, N, D> ConvolutionOps<S, T, DspVec<S, T, N, D>> for DspVec<S, T, N, D>
             let is_complex = self.is_complex();
             let source_len = self.len();
             let imp_resp_len = impulse_response.len();
-            let mut target = buffer.get(source_len);
+            let mut target = buffer.borrow(source_len);
             let range_done = {
                 let source = self.data.to_slice();
                 let imp_resp = impulse_response.data.to_slice();
                 let mut target = target.to_slice_mut();
-                T::gpu_convolve_vector(
-                    is_complex,
-                    &source[0..source_len],
-                    &mut target[0..source_len],
-                    &imp_resp[0..imp_resp_len])
+                T::gpu_convolve_vector(is_complex,
+                                       &source[0..source_len],
+                                       &mut target[0..source_len],
+                                       &imp_resp[0..imp_resp_len])
             };
             match range_done {
-                None =>
-                    buffer.free(target), // The GPU code can't handle this go to the next cases
+                None => (), // The GPU code can't handle this go to the next cases
                 Some(range) => {
                     self.convolve_vector_range(target.to_slice_mut(), impulse_response, range);
-                    mem::swap(&mut target, &mut self.data);
-                    buffer.free(target);
+                    target.trade(&mut self.data);
                     return Ok(());
                 }
             };
@@ -485,13 +518,12 @@ impl<S, T, N, D> ConvolutionOps<S, T, DspVec<S, T, N, D>> for DspVec<S, T, N, D>
             && impulse_response.len() > 15
             // Overlap-discard creates a lot of copies of the size of impulse_response
             // and that only seems to be worthwhile if the vector is long enough
-            && self.len() > 10 * impulse_response.len()
-            && self.is_complex() {
+            && self.len() > 10 * impulse_response.len() && self.is_complex() {
             let fft_len = next_power_of_two(impulse_response.len());
             return self.overlap_discard(buffer, impulse_response, fft_len);
         }
 
-        self.convolve_vector_scalar(buffer, impulse_response);
+        self.convolve_signal_scalar(buffer, impulse_response);
         return Ok(());
     }
 }
@@ -556,7 +588,6 @@ mod tests {
     use super::super::{WrappingIterator, ReverseWrappingIterator};
     use super::super::super::*;
     use conv_types::*;
-    use RealNumber;
     use num_complex::Complex32;
     use std::fmt::Debug;
 
@@ -686,7 +717,7 @@ mod tests {
         let argument = (&real[..]).interleave_to_complex_time_vec(&&imag[..]).unwrap();
         assert_eq!(time.points(), argument.points());
         let mut buffer = SingleBuffer::new();
-        time.convolve_vector(&mut buffer, &argument).unwrap();
+        time.convolve_signal(&mut buffer, &argument).unwrap();
         assert_eq!(time.points(), LEN);
         let result = time.magnitude();
         assert_eq!(result.points(), LEN);
@@ -735,7 +766,7 @@ mod tests {
         let b = vec![15.0, 14.0, 13.0, 12.0, 11.0, 10.0, 9.0, 8.0, 7.0, 6.0].to_complex_time_vec();
         let mut buffer = SingleBuffer::new();
         let mut conv = a.clone();
-        conv.convolve_vector(&mut buffer, &b).unwrap();
+        conv.convolve_signal(&mut buffer, &b).unwrap();
         let mut a = a.fft(&mut buffer);
         let b = b.fft(&mut buffer);
         a.mul(&b).unwrap();
@@ -752,7 +783,7 @@ mod tests {
         let mut buffer = SingleBuffer::new();
         let mut a = a.to_complex().unwrap();
         let b = b.to_complex().unwrap();
-        a.convolve_vector(&mut buffer, &b).unwrap();
+        a.convolve_signal(&mut buffer, &b).unwrap();
         let a = a.magnitude();
         let exp = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0];
         assert_eq_tol(&a[..], &exp, 1e-4);
@@ -765,7 +796,7 @@ mod tests {
         let b = vec![0.0, 0.0, 1.0].to_real_time_vec();
         let mut a = a.to_complex().unwrap();
         let b = b.to_complex().unwrap();
-        a.convolve_vector(&mut buffer, &b).unwrap();
+        a.convolve_signal(&mut buffer, &b).unwrap();
         let a = a.magnitude();
         let exp = [9.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0];
         assert_eq_tol(&a[..], &exp, 1e-4);
@@ -779,7 +810,7 @@ mod tests {
         let a = a.to_complex().unwrap();
         let b = b.to_complex().unwrap();
         let mut conv = a.clone();
-        conv.convolve_vector(&mut buffer, &b).unwrap();
+        conv.convolve_signal(&mut buffer, &b).unwrap();
         let mut a = a.fft(&mut buffer);
         let b = b.fft(&mut buffer);
         a.mul(&b).unwrap();
@@ -799,7 +830,7 @@ mod tests {
         let a = a.to_complex().unwrap();
         let b = b.to_complex().unwrap();
         let mut conv = a.clone();
-        conv.convolve_vector(&mut buffer, &b).unwrap();
+        conv.convolve_signal(&mut buffer, &b).unwrap();
         let mut a = a.fft(&mut buffer);
         let b = b.fft(&mut buffer);
         a.mul(&b).unwrap();
@@ -813,7 +844,7 @@ mod tests {
 
     #[test]
     fn overlap_discard_test() {
-        let a: Vec<f32> = (0..100).map(|x|x as f32).collect();
+        let a: Vec<f32> = (0..100).map(|x| x as f32).collect();
         let b: Vec<f32> = vec![0.1, 0.2, 0.3, 0.5, 0.1, 0.2];
         let a = a.to_real_time_vec();
         let b = b.to_real_time_vec();
@@ -821,10 +852,20 @@ mod tests {
         let a = a.to_complex().unwrap();
         let b = b.to_complex().unwrap();
         let mut conv = a.clone();
-        conv.convolve_vector_scalar(&mut buffer, &b);
-
+        conv.convolve_signal(&mut buffer, &b).unwrap();
         let mut overlap_discard = a;
         overlap_discard.overlap_discard(&mut buffer, &b, 0).unwrap();
         assert_eq_tol(&overlap_discard[..], &conv[..], 1e-4);
     }
+
+    /// This test triggered an index out of range error in the past
+    #[test]
+    fn index_out_of_range_error_test() {
+        let data: Vec<f32> = vec![0.0; 5000];
+        let mut vec = data.clone().to_real_time_vec();
+        let mut buffer = SingleBuffer::new();
+        let sinc = SincFunction::new();
+        vec.convolve(&mut buffer, &sinc as &RealImpulseResponse<f32>, 1.0, 12);
+        assert_eq_tol(&data[..], &vec[..], 1e-4);
+    }
 }
