@@ -1,7 +1,7 @@
 //! This module defines the basic vector trait and indexers.
-use {RealNumber, array_to_complex, array_to_complex_mut};
-use super::{DspVec, NumberSpace, ComplexNumberSpace,
-            Domain, DataDomain, ToSlice, ToSliceMut, ErrorReason, VoidResult, TypeMetaData};
+use {array_to_complex, array_to_complex_mut};
+use super::{DspVec, NumberSpace, ComplexNumberSpace, Buffer, BufferBorrow, Domain, DataDomain,
+            ToSlice, ToSliceMut, ErrorReason, VoidResult, TypeMetaData};
 use multicore_support::MultiCoreSettings;
 use std::ops::*;
 use numbers::*;
@@ -54,6 +54,16 @@ pub trait ResizeOps {
     fn resize(&mut self, len: usize) -> VoidResult;
 }
 
+/// Operations to resize a data type.
+pub trait ResizeBufferedOps<S: ToSliceMut<T>, T: RealNumber> {
+    /// Changes `self.len()`.
+    /// If `self.is_complex()` is true then `len` must be an even number.
+    /// `len > self.alloc_len()` is only possible if the underlying storage or the buffer
+    /// supports resizing.
+    fn resize_b<B>(&mut self, buffer: &mut B, len: usize) -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>;
+}
+
 /// A trait for vector types.
 pub trait Vector<T>: MetaData + ResizeOps
     where T: RealNumber
@@ -109,7 +119,8 @@ pub trait Vector<T>: MetaData + ResizeOps
 pub trait GetMetaData<T, N, D>
     where T: RealNumber,
           N: NumberSpace,
-          D: Domain {
+          D: Domain
+{
     /// Gets a copy of the vector meta data. This can be used to create
     /// new types with the same meta data.
     fn get_meta_data(&self) -> TypeMetaData<T, N, D>;
@@ -165,6 +176,35 @@ impl<S, T, N, D> ResizeOps for DspVec<S, T, N, D>
     }
 }
 
+impl<S, T, N, D> ResizeBufferedOps<S, T> for DspVec<S, T, N, D>
+    where S: ToSliceMut<T>,
+          T: RealNumber,
+          N: NumberSpace,
+          D: Domain
+{
+    fn resize_b<B>(&mut self, buffer: &mut B, len: usize) -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>
+    {
+        if self.is_complex() && len % 2 != 0 {
+            return Err(ErrorReason::InputMustHaveAnEvenLength);
+        }
+        let res = self.resize(len);
+        match res {
+            Ok(_) => Ok(()),
+            Err(_) => {
+                let orig_len = self.len();
+                let mut temp = buffer.borrow(len);
+                {
+                    &mut temp[0..orig_len].clone_from_slice(&self[..]);
+                }
+                temp.trade(&mut self.data);
+                self.valid_len = len;
+                Ok(())
+            }
+        }
+    }
+}
+
 impl<S, T, N, D> Vector<T> for DspVec<S, T, N, D>
     where S: ToSlice<T>,
           T: RealNumber,
@@ -208,13 +248,14 @@ impl<S, T, N, D> GetMetaData<T, N, D> for DspVec<S, T, N, D>
     where S: ToSlice<T>,
           T: RealNumber,
           N: NumberSpace,
-          D: Domain {
+          D: Domain
+{
     fn get_meta_data(&self) -> TypeMetaData<T, N, D> {
         TypeMetaData {
             number_space: self.number_space.clone(),
             domain: self.domain.clone(),
             delta: self.delta,
-            multicore_settings: self.multicore_settings
+            multicore_settings: self.multicore_settings,
         }
     }
 }
